@@ -7,8 +7,8 @@ import (
 )
 
 type cdCmdConfig struct {
-	Command string   `mapstructure:"command"`
-	Args    []string `mapstructure:"args"`
+	Command string   `json:"command" mapstructure:"command" yaml:"command"`
+	Args    []string `json:"args" mapstructure:"args" yaml:"args"`
 }
 
 func (c *Config) newCDCmd() *cobra.Command {
@@ -19,12 +19,12 @@ func (c *Config) newCDCmd() *cobra.Command {
 		Example: example("cd"),
 		RunE:    c.runCDCmd,
 		Args:    cobra.NoArgs,
-		Annotations: map[string]string{
-			createSourceDirectoryIfNeeded: "true",
-			doesNotRequireValidConfig:     "true",
-			requiresWorkingTree:           "true",
-			runsCommands:                  "true",
-		},
+		Annotations: newAnnotations(
+			createSourceDirectoryIfNeeded,
+			doesNotRequireValidConfig,
+			requiresWorkingTree,
+			runsCommands,
+		),
 	}
 
 	return cdCmd
