@@ -66,68 +66,68 @@ type purgeOptions struct {
 }
 
 type templateConfig struct {
-	Options []string `mapstructure:"options"`
+	Options []string `json:"options" mapstructure:"options" yaml:"options"`
 }
 
 type warningsConfig struct {
-	ConfigFileTemplateHasChanged bool `mapstructure:"configFileTemplateHasChanged"`
+	ConfigFileTemplateHasChanged bool `json:"configFileTemplateHasChanged" mapstructure:"configFileTemplateHasChanged" yaml:"configFileTemplateHasChanged"` //nolint:lll
 }
 
 // ConfigFile contains all data settable in the config file.
 type ConfigFile struct {
 	// Global configuration.
-	CacheDirAbsPath    chezmoi.AbsPath                 `mapstructure:"cacheDir"`
-	Color              autoBool                        `mapstructure:"color"`
-	Data               map[string]any                  `mapstructure:"data"`
-	Format             writeDataFormat                 `mapstructure:"format"`
-	DestDirAbsPath     chezmoi.AbsPath                 `mapstructure:"destDir"`
-	Interpreters       map[string]*chezmoi.Interpreter `mapstructure:"interpreters"`
-	Mode               chezmoi.Mode                    `mapstructure:"mode"`
-	Pager              string                          `mapstructure:"pager"`
-	PINEntry           pinEntryConfig                  `mapstructure:"pinentry"`
-	Progress           bool                            `mapstructure:"progress"`
-	Safe               bool                            `mapstructure:"safe"`
-	ScriptEnv          map[string]string               `mapstructure:"scriptEnv"`
-	ScriptTempDir      chezmoi.AbsPath                 `mapstructure:"scriptTempDir"`
-	SourceDirAbsPath   chezmoi.AbsPath                 `mapstructure:"sourceDir"`
-	Template           templateConfig                  `mapstructure:"template"`
-	TextConv           textConv                        `mapstructure:"textConv"`
-	Umask              fs.FileMode                     `mapstructure:"umask"`
-	UseBuiltinAge      autoBool                        `mapstructure:"useBuiltinAge"`
-	UseBuiltinGit      autoBool                        `mapstructure:"useBuiltinGit"`
-	Verbose            bool                            `mapstructure:"verbose"`
-	Warnings           warningsConfig                  `mapstructure:"warnings"`
-	WorkingTreeAbsPath chezmoi.AbsPath                 `mapstructure:"workingTree"`
+	CacheDirAbsPath    chezmoi.AbsPath                 `json:"cacheDir" mapstructure:"cacheDir" yaml:"cacheDir"`
+	Color              autoBool                        `json:"color" mapstructure:"color" yaml:"color"`
+	Data               map[string]any                  `json:"data" mapstructure:"data" yaml:"data"`
+	Format             writeDataFormat                 `json:"format" mapstructure:"format" yaml:"format"`
+	DestDirAbsPath     chezmoi.AbsPath                 `json:"destDir" mapstructure:"destDir" yaml:"destDir"`
+	Interpreters       map[string]*chezmoi.Interpreter `json:"interpreters" mapstructure:"interpreters" yaml:"interpreters"` //nolint:lll
+	Mode               chezmoi.Mode                    `json:"mode" mapstructure:"mode" yaml:"mode"`
+	Pager              string                          `json:"pager" mapstructure:"pager" yaml:"pager"`
+	PINEntry           pinEntryConfig                  `json:"pinentry" mapstructure:"pinentry" yaml:"pinentry"`
+	Progress           bool                            `json:"progress" mapstructure:"progress" yaml:"progress"`
+	Safe               bool                            `json:"safe" mapstructure:"safe" yaml:"safe"`
+	ScriptEnv          map[string]string               `json:"scriptEnv" mapstructure:"scriptEnv" yaml:"scriptEnv"`
+	ScriptTempDir      chezmoi.AbsPath                 `json:"scriptTempDir" mapstructure:"scriptTempDir" yaml:"scriptTempDir"` //nolint:lll
+	SourceDirAbsPath   chezmoi.AbsPath                 `json:"sourceDir" mapstructure:"sourceDir" yaml:"sourceDir"`
+	Template           templateConfig                  `json:"template" mapstructure:"template" yaml:"template"`
+	TextConv           textConv                        `json:"textConv" mapstructure:"textConv" yaml:"textConv"`
+	Umask              fs.FileMode                     `json:"umask" mapstructure:"umask" yaml:"umask"`
+	UseBuiltinAge      autoBool                        `json:"useBuiltinAge" mapstructure:"useBuiltinAge" yaml:"useBuiltinAge"` //nolint:lll
+	UseBuiltinGit      autoBool                        `json:"useBuiltinGit" mapstructure:"useBuiltinGit" yaml:"useBuiltinGit"` //nolint:lll
+	Verbose            bool                            `json:"verbose" mapstructure:"verbose" yaml:"verbose"`
+	Warnings           warningsConfig                  `json:"warnings" mapstructure:"warnings" yaml:"warnings"`
+	WorkingTreeAbsPath chezmoi.AbsPath                 `json:"workingTree" mapstructure:"workingTree" yaml:"workingTree"`
 
 	// Password manager configurations.
-	AWSSecretsManager awsSecretsManagerConfig `mapstructure:"awsSecretsManager"`
-	Bitwarden         bitwardenConfig         `mapstructure:"bitwarden"`
-	Gopass            gopassConfig            `mapstructure:"gopass"`
-	Keepassxc         keepassxcConfig         `mapstructure:"keepassxc"`
-	Keeper            keeperConfig            `mapstructure:"keeper"`
-	Lastpass          lastpassConfig          `mapstructure:"lastpass"`
-	Onepassword       onepasswordConfig       `mapstructure:"onepassword"`
-	Pass              passConfig              `mapstructure:"pass"`
-	Passhole          passholeConfig          `mapstructure:"passhole"`
-	Secret            secretConfig            `mapstructure:"secret"`
-	Vault             vaultConfig             `mapstructure:"vault"`
+	AWSSecretsManager awsSecretsManagerConfig `json:"awsSecretsManager" mapstructure:"awsSecretsManager" yaml:"awsSecretsManager"` //nolint:lll
+	Bitwarden         bitwardenConfig         `json:"bitwarden" mapstructure:"bitwarden" yaml:"bitwarden"`
+	Gopass            gopassConfig            `json:"gopass" mapstructure:"gopass" yaml:"gopass"`
+	Keepassxc         keepassxcConfig         `json:"keepassxc" mapstructure:"keepassxc" yaml:"keepassxc"`
+	Keeper            keeperConfig            `json:"keeper" mapstructure:"keeper" yaml:"keeper"`
+	Lastpass          lastpassConfig          `json:"lastpass" mapstructure:"lastpass" yaml:"lastpass"`
+	Onepassword       onepasswordConfig       `json:"onepassword" mapstructure:"onepassword" yaml:"onepassword"`
+	Pass              passConfig              `json:"pass" mapstructure:"pass" yaml:"pass"`
+	Passhole          passholeConfig          `json:"passhole" mapstructure:"passhole" yaml:"passhole"`
+	Secret            secretConfig            `json:"secret" mapstructure:"secret" yaml:"secret"`
+	Vault             vaultConfig             `json:"vault" mapstructure:"vault" yaml:"vault"`
 
 	// Encryption configurations.
-	Encryption string                `json:"encryption" mapstructure:"encryption"`
-	Age        chezmoi.AgeEncryption `json:"age" mapstructure:"age"`
-	GPG        chezmoi.GPGEncryption `json:"gpg" mapstructure:"gpg"`
+	Encryption string                `json:"encryption" mapstructure:"encryption" yaml:"encryption"`
+	Age        chezmoi.AgeEncryption `json:"age" mapstructure:"age" yaml:"age"`
+	GPG        chezmoi.GPGEncryption `json:"gpg" mapstructure:"gpg" yaml:"gpg"`
 
 	// Command configurations.
-	Add        addCmdConfig        `mapstructure:"add"`
-	CD         cdCmdConfig         `mapstructure:"cd"`
-	Completion completionCmdConfig `mapstructure:"completion"`
-	Diff       diffCmdConfig       `mapstructure:"diff"`
-	Edit       editCmdConfig       `mapstructure:"edit"`
-	Git        gitCmdConfig        `mapstructure:"git"`
-	Merge      mergeCmdConfig      `mapstructure:"merge"`
-	Status     statusCmdConfig     `mapstructure:"status"`
-	Update     updateCmdConfig     `mapstructure:"update"`
-	Verify     verifyCmdConfig     `mapstructure:"verify"`
+	Add        addCmdConfig        `json:"add" mapstructure:"add" yaml:"add"`
+	CD         cdCmdConfig         `json:"cd" mapstructure:"cd" yaml:"cd"`
+	Completion completionCmdConfig `json:"completion" mapstructure:"completion" yaml:"completion"`
+	Diff       diffCmdConfig       `json:"diff" mapstructure:"diff" yaml:"diff"`
+	Edit       editCmdConfig       `json:"edit" mapstructure:"edit" yaml:"edit"`
+	Git        gitCmdConfig        `json:"git" mapstructure:"git" yaml:"git"`
+	Merge      mergeCmdConfig      `json:"merge" mapstructure:"merge" yaml:"merge"`
+	Status     statusCmdConfig     `json:"status" mapstructure:"status" yaml:"status"`
+	Update     updateCmdConfig     `json:"update" mapstructure:"update" yaml:"update"`
+	Verify     verifyCmdConfig     `json:"verify" mapstructure:"verify" yaml:"verify"`
 }
 
 // A Config represents a configuration.
@@ -335,12 +335,14 @@ func newConfig(options ...configOption) (*Config, error) {
 	}
 
 	for key, value := range map[string]any{
-		"awsSecretsManager":        c.awsSecretsManagerTemplateFunc,
-		"awsSecretsManagerRaw":     c.awsSecretsManagerRawTemplateFunc,
-		"bitwarden":                c.bitwardenTemplateFunc,
-		"bitwardenAttachment":      c.bitwardenAttachmentTemplateFunc,
-		"bitwardenFields":          c.bitwardenFieldsTemplateFunc,
-		"comment":                  c.commentTemplateFunc,
+		"awsSecretsManager":    c.awsSecretsManagerTemplateFunc,
+		"awsSecretsManagerRaw": c.awsSecretsManagerRawTemplateFunc,
+		"bitwarden":            c.bitwardenTemplateFunc,
+		"bitwardenAttachment":  c.bitwardenAttachmentTemplateFunc,
+		"bitwardenFields":      c.bitwardenFieldsTemplateFunc,
+		"comment":              c.commentTemplateFunc,
+		// The completion template function is added in persistentPreRunRootE as
+		// it needs a *cobra.Command, which we don't yet have.
 		"decrypt":                  c.decryptTemplateFunc,
 		"encrypt":                  c.encryptTemplateFunc,
 		"eqFold":                   c.eqFoldTemplateFunc,
@@ -382,6 +384,7 @@ func newConfig(options ...configOption) (*Config, error) {
 		"replaceAllRegex":          c.replaceAllRegexTemplateFunc,
 		"secret":                   c.secretTemplateFunc,
 		"secretJSON":               c.secretJSONTemplateFunc,
+		"setValueAtPath":           c.setValueAtPathTemplateFunc,
 		"stat":                     c.statTemplateFunc,
 		"toIni":                    c.toIniTemplateFunc,
 		"toToml":                   c.toTomlTemplateFunc,
@@ -688,22 +691,18 @@ func (c *Config) createConfigFile(filename chezmoi.RelPath, data []byte) ([]byte
 	}
 	chezmoi.RecursiveMerge(funcMap, initTemplateFuncs)
 
-	t, err := chezmoi.ParseTemplate(filename.String(), data, funcMap, chezmoi.TemplateOptions{
+	tmpl, err := chezmoi.ParseTemplate(filename.String(), data, funcMap, chezmoi.TemplateOptions{
 		Options: append([]string(nil), c.Template.Options...),
 	})
 	if err != nil {
 		return nil, err
 	}
 
-	builder := strings.Builder{}
 	templateData := c.getTemplateDataMap()
 	if c.init.data {
 		chezmoi.RecursiveMerge(templateData, c.Data)
 	}
-	if err = t.Execute(&builder, templateData); err != nil {
-		return nil, err
-	}
-	return []byte(builder.String()), nil
+	return tmpl.Execute(templateData)
 }
 
 // defaultConfigFile returns the default config file according to the XDG Base
@@ -1294,11 +1293,11 @@ func (c *Config) gitAutoCommit(status *git.Status) error {
 	if err != nil {
 		return err
 	}
-	commitMessage := strings.Builder{}
-	if err := commitMessageTmpl.Execute(&commitMessage, status); err != nil {
+	commitMessage, err := commitMessageTmpl.Execute(status)
+	if err != nil {
 		return err
 	}
-	return c.run(c.WorkingTreeAbsPath, c.Git.Command, []string{"commit", "--message", commitMessage.String()})
+	return c.run(c.WorkingTreeAbsPath, c.Git.Command, []string{"commit", "--message", string(commitMessage)})
 }
 
 // gitAutoPush pushes all changes to the remote if status is not empty.
@@ -1398,6 +1397,7 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 		c.newApplyCmd(),
 		c.newArchiveCmd(),
 		c.newCatCmd(),
+		c.newCatConfigCmd(),
 		c.newCDCmd(),
 		c.newChattrCmd(),
 		c.newCompletionCmd(),
@@ -1406,6 +1406,7 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 		c.newDiffCmd(),
 		c.newDoctorCmd(),
 		c.newDumpCmd(),
+		c.newDumpConfigCmd(),
 		c.newEditCmd(),
 		c.newEditConfigCmd(),
 		c.newEncryptCommand(),
@@ -1448,16 +1449,18 @@ func (c *Config) newRootCmd() (*cobra.Command, error) {
 func (c *Config) newDiffSystem(s chezmoi.System, w io.Writer, dirAbsPath chezmoi.AbsPath) chezmoi.System {
 	if c.Diff.useBuiltinDiff || c.Diff.Command == "" {
 		options := &chezmoi.GitDiffSystemOptions{
-			Color:        c.Color.Value(c.colorAutoFunc),
-			Filter:       chezmoi.NewEntryTypeFilter(c.Diff.include.Bits(), c.Diff.Exclude.Bits()),
-			Reverse:      c.Diff.Reverse,
-			TextConvFunc: c.TextConv.convert,
+			Color:          c.Color.Value(c.colorAutoFunc),
+			Filter:         chezmoi.NewEntryTypeFilter(c.Diff.include.Bits(), c.Diff.Exclude.Bits()),
+			Reverse:        c.Diff.Reverse,
+			ScriptContents: c.Diff.ScriptContents,
+			TextConvFunc:   c.TextConv.convert,
 		}
 		return chezmoi.NewGitDiffSystem(s, w, dirAbsPath, options)
 	}
 	options := &chezmoi.ExternalDiffSystemOptions{
-		Filter:  chezmoi.NewEntryTypeFilter(c.Diff.include.Bits(), c.Diff.Exclude.Bits()),
-		Reverse: c.Diff.Reverse,
+		Filter:         chezmoi.NewEntryTypeFilter(c.Diff.include.Bits(), c.Diff.Exclude.Bits()),
+		Reverse:        c.Diff.Reverse,
+		ScriptContents: c.Diff.ScriptContents,
 	}
 	return chezmoi.NewExternalDiffSystem(s, c.Diff.Command, c.Diff.Args, c.DestDirAbsPath, options)
 }
@@ -1512,11 +1515,13 @@ func (c *Config) newSourceState(
 
 // persistentPostRunRootE performs post-run actions for the root command.
 func (c *Config) persistentPostRunRootE(cmd *cobra.Command, args []string) error {
+	annotations := getAnnotations(cmd)
+
 	if err := c.persistentState.Close(); err != nil {
 		return err
 	}
 
-	if boolAnnotation(cmd, modifiesConfigFile) {
+	if annotations.hasTag(modifiesConfigFile) {
 		configFileContents, err := c.baseSystem.ReadFile(c.configFileAbsPath)
 		switch {
 		case errors.Is(err, fs.ErrNotExist):
@@ -1539,7 +1544,7 @@ func (c *Config) persistentPostRunRootE(cmd *cobra.Command, args []string) error
 		}
 	}
 
-	if boolAnnotation(cmd, modifiesSourceDirectory) {
+	if annotations.hasTag(modifiesSourceDirectory) {
 		var status *git.Status
 		if c.Git.AutoAdd || c.Git.AutoCommit || c.Git.AutoPush {
 			var err error
@@ -1598,6 +1603,18 @@ func (c *Config) pageOutputString(output, cmdPager string) error {
 
 // persistentPreRunRootE performs pre-run actions for the root command.
 func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error {
+	annotations := getAnnotations(cmd)
+
+	// Add the completion template function. This needs cmd, so we can't do it
+	// in newConfig.
+	c.addTemplateFunc("completion", func(shell string) string {
+		completion, err := completion(cmd, shell)
+		if err != nil {
+			panic(err)
+		}
+		return completion
+	})
+
 	// Enable CPU profiling if configured.
 	if !c.cpuProfile.Empty() {
 		f, err := os.Create(c.cpuProfile.String())
@@ -1632,7 +1649,7 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 	})
 
 	// Read the config file.
-	if boolAnnotation(cmd, doesNotRequireValidConfig) {
+	if annotations.hasTag(doesNotRequireValidConfig) {
 		if c.configFileAbsPathErr == nil {
 			_ = c.readConfig()
 		}
@@ -1684,10 +1701,10 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 	}
 
 	// Set up the persistent state.
-	switch {
-	case cmd.Annotations[persistentStateMode] == persistentStateModeEmpty:
+	switch persistentStateMode := annotations.persistentStateMode(); {
+	case persistentStateMode == persistentStateModeEmpty:
 		c.persistentState = chezmoi.NewMockPersistentState()
-	case cmd.Annotations[persistentStateMode] == persistentStateModeReadOnly:
+	case persistentStateMode == persistentStateModeReadOnly:
 		persistentStateFileAbsPath, err := c.persistentStateFile()
 		if err != nil {
 			return err
@@ -1698,9 +1715,9 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 		if err != nil {
 			return err
 		}
-	case cmd.Annotations[persistentStateMode] == persistentStateModeReadMockWrite:
+	case persistentStateMode == persistentStateModeReadMockWrite:
 		fallthrough
-	case cmd.Annotations[persistentStateMode] == persistentStateModeReadWrite && c.dryRun:
+	case persistentStateMode == persistentStateModeReadWrite && c.dryRun:
 		persistentStateFileAbsPath, err := c.persistentStateFile()
 		if err != nil {
 			return err
@@ -1719,7 +1736,7 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 			return err
 		}
 		c.persistentState = dryRunPersistentState
-	case cmd.Annotations[persistentStateMode] == persistentStateModeReadWrite:
+	case persistentStateMode == persistentStateModeReadWrite:
 		persistentStateFileAbsPath, err := c.persistentStateFile()
 		if err != nil {
 			return err
@@ -1741,10 +1758,10 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 	// Set up the source and destination systems.
 	c.sourceSystem = c.baseSystem
 	c.destSystem = c.baseSystem
-	if !boolAnnotation(cmd, modifiesDestinationDirectory) {
+	if !annotations.hasTag(modifiesDestinationDirectory) {
 		c.destSystem = chezmoi.NewReadOnlySystem(c.destSystem)
 	}
-	if !boolAnnotation(cmd, modifiesSourceDirectory) {
+	if !annotations.hasTag(modifiesSourceDirectory) {
 		c.sourceSystem = chezmoi.NewReadOnlySystem(c.sourceSystem)
 	}
 	if c.dryRun {
@@ -1761,21 +1778,21 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 	}
 
 	// Create the config directory if needed.
-	if boolAnnotation(cmd, requiresConfigDirectory) {
+	if annotations.hasTag(requiresConfigDirectory) {
 		if err := chezmoi.MkdirAll(c.baseSystem, c.configFileAbsPath.Dir(), fs.ModePerm); err != nil {
 			return err
 		}
 	}
 
 	// Create the source directory if needed.
-	if boolAnnotation(cmd, createSourceDirectoryIfNeeded) {
+	if annotations.hasTag(createSourceDirectoryIfNeeded) {
 		if err := chezmoi.MkdirAll(c.baseSystem, c.SourceDirAbsPath, fs.ModePerm); err != nil {
 			return err
 		}
 	}
 
 	// Verify that the source directory exists and is a directory, if needed.
-	if boolAnnotation(cmd, requiresSourceDirectory) {
+	if annotations.hasTag(requiresSourceDirectory) {
 		switch fileInfo, err := c.baseSystem.Stat(c.SourceDirAbsPath); {
 		case err == nil && fileInfo.IsDir():
 			// Do nothing.
@@ -1787,7 +1804,7 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 	}
 
 	// Create the runtime directory if needed.
-	if boolAnnotation(cmd, runsCommands) {
+	if annotations.hasTag(runsCommands) {
 		if runtime.GOOS == "linux" && c.bds.RuntimeDir != "" {
 			// Snap sets the $XDG_RUNTIME_DIR environment variable to
 			// /run/user/$uid/snap.$snap_name, but does not create this
@@ -1821,7 +1838,7 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 	}
 
 	// Create the working tree directory if needed.
-	if boolAnnotation(cmd, requiresWorkingTree) {
+	if annotations.hasTag(requiresWorkingTree) {
 		if _, err := c.SourceDirAbsPath.TrimDirPrefix(c.WorkingTreeAbsPath); err != nil {
 			return err
 		}
@@ -2384,8 +2401,9 @@ func newConfigFile(bds *xdg.BaseDirectorySpecification) ConfigFile {
 			recursive: true,
 		},
 		Diff: diffCmdConfig{
-			Exclude: chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
-			include: chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
+			Exclude:        chezmoi.NewEntryTypeSet(chezmoi.EntryTypesNone),
+			ScriptContents: true,
+			include:        chezmoi.NewEntryTypeSet(chezmoi.EntryTypesAll),
 		},
 		Edit: editCmdConfig{
 			Hardlink:    true,
