@@ -4,7 +4,6 @@
 package cmd
 
 import (
-	"io"
 	"io/fs"
 	"syscall"
 
@@ -15,12 +14,10 @@ const defaultEditor = "vi"
 
 var defaultInterpreters = make(map[string]*chezmoi.Interpreter)
 
-// enableVirtualTerminalProcessing does nothing on non-Windows systems.
-func enableVirtualTerminalProcessing(w io.Writer) (func() error, error) {
-	return nil, nil
+func fileInfoUID(info fs.FileInfo) int {
+	return int(info.Sys().(*syscall.Stat_t).Uid) //nolint:forcetypeassert
 }
 
-func fileInfoUID(info fs.FileInfo) int {
-	//nolint:forcetypeassert
-	return int(info.Sys().(*syscall.Stat_t).Uid)
+func windowsVersion() (map[string]any, error) {
+	return nil, nil
 }
